#pragma once

#include "Atlas.h"
#include "ofMain.h"

//uuid
#include "random_id.h"

#include "ofxMsdfgen.h"
#include <memory>

namespace ofxVariableLab {
class Layer {
    public:
        enum Type {
            MSDFGEN = 0,
            GPUFONT
        };
        struct Props {
            float x = 0;
            float y = 0;
            float rotation = 0;
            float fontSize_px = 24;
            float * color[4];
            bool mirror_x = false;
            float mirror_x_distance = 0;
            bool mirror_y = false;
            float mirror_y_distance = 0;
            float letterDelay = 0;
            string text = "abcdefghijklmnoprstuvqxyzABUIWERJSD";
        };
        struct Settings {
            uint32_t maxBufferSize = 100;
        };

        virtual void setup(const Settings & settings) = 0;
        virtual void update() = 0;
        virtual void draw(glm::vec3 position = glm::vec3(0, 0, 0)) const = 0;
        virtual void drawCharacter(const char character,
                                   glm::vec3 position = glm::vec3(0, 0, 0),
                                   ofxMsdfgen::FontVariation fontVariation = ofxMsdfgen::FontVariation()) const = 0;
        virtual void setProps(const Props & props) = 0;
        virtual const Props & getProps() const = 0;
        virtual void clearPropsBuffer() = 0;
        virtual void setId(const string & id) = 0;
        virtual const string & getId() = 0;
        virtual void setShader(shared_ptr <ofShader> _shader) = 0;
        virtual shared_ptr <ofShader> getShader() const = 0;
        virtual const Type & getType() const = 0;
};

}
